﻿using UnityEngine;
using UnityEngine.PostProcessing;
using System;
using System.Collections.Generic;
using SimpleJSON;
using MeshVR;

namespace VaMJapan
{	
	public class DesctopScreenshot : MVRScript
	{	
		private const int READY = 0;
		private const int SCREENSHOT = 1;
		private const int CLEANUP = 2;
		
		private const int resolutionDefaultIdx = 1;		
		private readonly int[] resolutionValue = new int[] { 1920,3840,7680,9600};
		private readonly string[] resolutionNames = new string[] {
			"1920px",
			"3840px",
			"7680px",
			"9600px",
        };
		private const int AspectRatioDefaultIdx = 0;		
		private double AspectRatioValue = 0;		
		private double AspectRatio;		
		private readonly double[] AspectRatioValues = new double[] {0,0.5625,0.666666,0.75,1};
		private readonly string[] AspectRatioNames = new string[] {
			"No Adjustment(Window)",
			"16:9",
			"3:2",
			"4:3",
			"1:1(9600 won't work)",
        };
		
		private readonly int[] msaaLevels = new int[] { 1, 2, 4, 8 };
		private readonly string[] msaaNames = new string[] { "Off", "2x", "4x", "8x" };		
		private int defaultMsaaIdx = 3;
		private int msaaLevel = 8;
		
		private RenderTexture screenshotTexture;
		private Camera previewCamera;
		private Camera screenshotCamera;
		private int state = 0;
		private int resolutionIdx = resolutionDefaultIdx;
		private bool navigationWasDisabled = false;
		
		private   JSONStorableFloat fov;
		protected JSONStorableStringChooser ScreenshotKey;

		private SuperController sc = SuperController.singleton;

		public override void Init()
		{

            fov= new JSONStorableFloat("FOV",  sc.monitorCameraFOVSlider.value, fovSliderCallback, 1f, 100f, true);
			RegisterFloat(fov);
			fovSlider = CreateSlider(fov);


        	List<string> resolutionList = new List<string>(resolutionNames);
			JSONStorableStringChooser resolutionChooser = new JSONStorableStringChooser("Longer Side", resolutionList, resolutionList[resolutionDefaultIdx], "Longer Side");
			resolutionChooser.setCallbackFunction += (string modeName) => {
				resolutionIdx = resolutionList.FindIndex((string entry) => { return entry == modeName; });
				if (resolutionIdx < 0 || resolutionIdx >= resolutionValue.Length )
					resolutionIdx = resolutionDefaultIdx;
			};
			CreateScrollablePopup(resolutionChooser, false);
			RegisterStringChooser(resolutionChooser);


			
			List<string> AspectRatioList = new List<string>(AspectRatioNames);
			JSONStorableStringChooser AspectRatioChooser = new JSONStorableStringChooser("Aspect Ratio Adjustment", AspectRatioList, AspectRatioList[AspectRatioDefaultIdx], "Aspect Ratio Adjustment");
			AspectRatioChooser.setCallbackFunction += (string modeName) => {
				int idx = AspectRatioList.FindIndex((string entry) => { return entry == modeName; });
				if (idx < 0 || idx >= AspectRatioValues.Length)
				idx = AspectRatioDefaultIdx;
				AspectRatioValue = AspectRatioValues[idx];
			};
			CreateScrollablePopup(AspectRatioChooser, false);
			RegisterStringChooser(AspectRatioChooser);
			AspectRatioChooser.setCallbackFunction(AspectRatioChooser.val);



			
			List<string> msaaList = new List<string>(msaaNames);
			JSONStorableStringChooser msaaChooser = new JSONStorableStringChooser("MSAA Override", msaaList, msaaList[defaultMsaaIdx], "MSAA Override");
			msaaChooser.setCallbackFunction += (string modeName) => {
				int idx = msaaList.FindIndex((string entry) => { return entry == modeName; });
				if (idx < 0 || idx >= msaaLevels.Length)
					idx = defaultMsaaIdx;
				msaaLevel = msaaLevels[idx];
			};
			CreateScrollablePopup(msaaChooser, false);
			RegisterStringChooser(msaaChooser);
			msaaChooser.setCallbackFunction(msaaChooser.val);

			screenshotCamera = sc.hiResScreenshotCamera;
			previewCamera = Instantiate<Camera>(screenshotCamera, screenshotCamera.transform);
			previewCamera.targetTexture = screenshotCamera.targetTexture;			
			previewCamera.enabled = false;
						
			screenshotCamera.targetTexture = null; // prevent regular VaM screenshot
			screenshotCamera.enabled = false; // prevent expensive rendering in the background
			
			state = READY;


			UIDynamicButton screenshotProcessButton = CreateButton("Take a Screenshot");
			screenshotProcessButton.button.onClick.AddListener(takeScreenshot);
			

			ScreenshotKey = new JSONStorableStringChooser("Screenshot Key", KeyCodes, "None", "Screenshot Key");
			RegisterStringChooser(ScreenshotKey);
			UIDynamicPopup dp = CreateScrollablePopup(ScreenshotKey,true);
			dp.popupPanelHeight = 1100f;
			if (ScreenshotKey.val =="None")
			{
				LoadShortCutKey();
			}


			UIDynamicButton savePresetButton = CreateButton("Save Key as Default",true);
			savePresetButton.button.onClick.AddListener(SaveShortCutKey);

            fovSliderCallback(fov);
		}

		private void OnEnable()
		{
			fovSliderCallback(fov);

            sc = sc;
			screenshotCamera = sc.hiResScreenshotCamera;
			previewCamera = Instantiate<Camera>(screenshotCamera, screenshotCamera.transform);
			previewCamera.targetTexture = screenshotCamera.targetTexture;			
			previewCamera.enabled = false;
						
			screenshotCamera.targetTexture = null; // prevent regular VaM screenshot
			screenshotCamera.enabled = false; // prevent expensive rendering in the background
			
			state = READY;
		}
		
		private void Update()
		{	
			GlobalSceneOptions gso = GlobalSceneOptions.singleton;
			if (ScreenshotKey.val != "None")
				{
						if (Input.GetKeyDown(ScreenshotKey.val))
						{
							takeScreenshot();
						}
				}

			switch (state)
			{
				case READY:
				{
					// enable cameras as needed
					

					bool active = sc.hiResScreenshotPreview.gameObject.activeSelf;
					screenshotCamera.enabled = false;

					previewCamera.enabled = active;
					previewCamera.fieldOfView = sc.hiResScreenShotCameraFOV;


					if (!active)
						return;

					if (screenshotTexture != null)
						{
							screenshotTexture.Release();
							Destroy(screenshotTexture);
						}
			
						RenderTextureDescriptor descriptor = previewCamera.targetTexture.descriptor;	
						
						int ScreenWidth = Screen.width;
						int ScreenHeight = Screen.height;
						int Longer = Math.Max(ScreenWidth,ScreenHeight);
						
						
							if (Longer == ScreenWidth)
							{
								if (AspectRatioValue == 0)
								{
									AspectRatio = (float)ScreenHeight /ScreenWidth;
								}else
								{
									AspectRatio = AspectRatioValue;
								}
							
								descriptor.width = resolutionValue[resolutionIdx];
								descriptor.height = (int)Math.Ceiling(AspectRatio *resolutionValue[resolutionIdx]);
							}else
							{
								if (AspectRatioValue == 0)
								{
									AspectRatioValue = (float)ScreenWidth /ScreenHeight;
								}else
								{
									AspectRatio = AspectRatioValue;
								}
						
								descriptor.width = (int)Math.Ceiling(AspectRatio *resolutionValue[resolutionIdx]);
								descriptor.height = resolutionValue[resolutionIdx];
							}
					
					
						descriptor.msaaSamples = msaaLevel;
						screenshotTexture = new RenderTexture(descriptor);
						screenshotTexture.Create();

					
					if (!screenshotProcess&&(!sc.GetRightSelect() && !sc.GetLeftSelect() && !sc.GetMouseSelect()))return;
				
					// init screenshot sequence
					screenshotCamera.targetTexture = screenshotTexture;			
					screenshotCamera.enabled = true;
					state = SCREENSHOT;
					break;
				}
				case SCREENSHOT:
				{	
					// fake user input to cause SuperController.ProcessHiResScreenshot do run
					navigationWasDisabled = gso.disableNavigation;
					gso.disableNavigation = true;
					sc.SetLeftSelect(); 
					state = CLEANUP;
					break;
				}
				case CLEANUP:
				{				
					// cleanup
					gso.disableNavigation = navigationWasDisabled;
					screenshotCamera.targetTexture = null;
					screenshotCamera.enabled = false;
					state = READY;
					if(screenshotProcess)
					{
						screenshotProcess = false;
						sc.SelectModeOff();
						if(UIvisiblity)sc.ShowMainHUD();
					}

					break;
				}
			}
		}
		
		private void OnDisable()
		{
			  sc.monitorCameraFOV = sc.monitorCameraFOVSlider.value;
			if (screenshotTexture != null)
			{
				screenshotTexture.Release();
				Destroy(screenshotTexture);
				screenshotTexture = null;
			}
			
			screenshotCamera.targetTexture = previewCamera.targetTexture;
			screenshotCamera.enabled = sc.hiResScreenshotPreview.gameObject.activeSelf;
			screenshotCamera = null;
			
			Destroy(previewCamera);
			previewCamera = null;
		}

		protected UIDynamicSlider fovSlider;
		protected void fovSliderCallback(JSONStorableFloat fov) {
		  sc.monitorCameraFOV = sc.hiResScreenShotCameraFOV = fov.val;
		}

		private bool screenshotProcess;
		private bool UIvisiblity;
		

		private void takeScreenshot(){
				if (sc.mainHUD.gameObject.active){
					UIvisiblity = true;
				}else{
					UIvisiblity = false;
				}
			screenshotProcess = true;
			sc.SelectModeScreenshot();
			fov.SetVal(fov.val+0.01f);
			fov.SetVal(fov.val-0.01f);	
		}


		private void SaveShortCutKey(){
            SimpleJSON.JSONClass Settings = new SimpleJSON.JSONClass();
			Settings.Add("Screenshot Key", new SimpleJSON.JSONData(ScreenshotKey.val));
			sc.SaveJSON(Settings, "Custom/Scripts/VaMJapan/DSsetting.json");
            SuperController.LogMessage("Screenshot Key was Saved");
		}

		private void LoadShortCutKey(){
            SimpleJSON.JSONClass LoadSettings = new SimpleJSON.JSONClass();
			LoadSettings = (SimpleJSON.JSONClass)sc.LoadJSON("Custom/Scripts/VaMJapan/DSsetting.json");
			if (LoadSettings != null)
            {
                ScreenshotKey.val = LoadSettings["Screenshot Key"];
			}
		}
List<string> KeyCodes = new List<string>() {
		"None",
		"space",
		"return",
		"up",
		"down",
		"left",
		"right",
		"left shift",
		"right shift",
		"left ctrl",
		"right ctrl",
		"left alt",
		"right alt",
		"tab",
		"backspace",
		"f2",
		"f3",
		"f4",
		"f5",
		"f6",
		"f7",
		"f8",
		"f9",
		"f10",
		"f11",
		"f12",
		"0",
		"1",
		"2",
		"3",
		"4",
		"5",
		"6",
		"7",
		"8",
		"9",
		"[0]",
		"[1]",
		"[2]",
		"[3]",
		"[4]",
		"[5]",
		"[6]",
		"[7]",
		"[8]",
		"[9]",
		"b",
		"c",
		"f",
		"g",
		"h",
		"i",
		"j",
		"k",
		"l",
		"o",
		"q",
		"r",
		"v",
		"y",
		"insert",
		"delete",
		"home",
		"end",
		"page up",
		"page down",
		"numlock",
		"caps lock",
		"scroll lock",
		"pause",
		};
    }

	
}